--[[
RP_CombatDice r2.2 - RP Dice Combat System.
Copyright (C) 2012-2013 Michel R Vaillancourt - walin_warlock_wra@wolfstar.ca

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]--

RP_CombatDice = LibStub("AceAddon-3.0"):NewAddon("RP Combat Dice", "AceConsole-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local options = {
	handler = RP_CombatDice,
	type = 'group',
	args = {
    heading_item={
      name="Character Skills",
      type="header"
    },
    description_item={
      name="Check-mark all skills your character is agreed to have in RP adventures",
      type="description",
      fontSize="medium"
    },
    melee_combat={
      name="Melee Combat",
      desc="Indicates if the character can fight well in close combat",
      type="toggle",
      set = "SetRPCD_CharIsMeleeCombat",
      get = "GetRPCD_CharIsMeleeCombat"
    },
    ranged_combat={
      name="Ranged Combat",
      desc="Indicates if the character can fight well in ranged battle",
      type="toggle",
      set = "SetRPCD_CharIsRangedCombat",
      get = "GetRPCD_CharIsRangedCombat"
    },
    spell_combat={
      name="Spell Combat",
      desc="Indicates if the character can fight well with spells in battle",
      type="toggle",
      set = "SetRPCD_CharIsSpellCombat",
      get = "GetRPCD_CharIsSpellCombat"
    },
    healer={
      name="Healing",
      desc="Indicates if the character can function as a healer",
      type="toggle",
      set = "SetRPCD_CharIsHealer",
      get = "GetRPCD_CharIsHealer"
    },
    lore={
      name="Lore",
      desc="Indicates if the character can provide insight and research into history and lore",
      type="toggle",
      set = "SetRPCD_CharIsLore",
      get = "GetRPCD_CharIsLore"
    },
    well_rounded={
      name="Well Rounded",
      desc="Indicates if the character is well rounded, having a solid base in all skills",
      type="toggle",
      set = "SetRPCD_CharIsWellRounded",
      get = "GetRPCD_CharIsWellRounded"
    }
	}
}

local defaults = {
	global = {
		RPCDsaidhelp  = 0,
		RPCD_CharIsMeleeCombat = true,
    RPCD_CharIsRangedCombat = false,
    RPCD_CharIsSpellCombat = false,
		RPCD_CharIsHealer = false,
		RPCD_CharIsLore = false,
		RPCD_CharIsWellRounded = false
	}
}

--get/set code
--
function RP_CombatDice:GetRPCD_CharIsMeleeCombat()
  return self.db.global.RPCD_CharIsMeleeCombat
end

function RP_CombatDice:SetRPCD_CharIsMeleeCombat(i, newValue)
  self.db.global.RPCD_CharIsMeleeCombat = newValue
  RP_CombatDice:SaySetSkills()
end

--
function RP_CombatDice:GetRPCD_CharIsRangedCombat()
	return self.db.global.RPCD_CharIsRangedCombat
end

function RP_CombatDice:SetRPCD_CharIsRangedCombat(i, newValue)
	self.db.global.RPCD_CharIsRangedCombat = newValue
	RP_CombatDice:SaySetSkills()
end

--
function RP_CombatDice:GetRPCD_CharIsSpellCombat()
  return self.db.global.RPCD_CharIsSpellCombat
end

function RP_CombatDice:SetRPCD_CharIsSpellCombat(i, newValue)
  self.db.global.RPCD_CharIsSpellCombat = newValue
  RP_CombatDice:SaySetSkills()
end

--
function RP_CombatDice:GetRPCD_CharIsHealer()
	return self.db.global.RPCD_CharIsHealer
end

function RP_CombatDice:SetRPCD_CharIsHealer(i, newValue)
	self.db.global.RPCD_CharIsHealer = newValue
	RP_CombatDice:SaySetSkills()
end

--
function RP_CombatDice:GetRPCD_CharIsLore()
	return self.db.global.RPCD_CharIsLore
end

function RP_CombatDice:SetRPCD_CharIsLore(i, newValue)
	self.db.global.RPCD_CharIsLore = newValue
	RP_CombatDice:SaySetSkills()
end

--
function RP_CombatDice:GetRPCD_CharIsWellRounded()
	return self.db.global.RPCD_CharIsWellRounded
end

function RP_CombatDice:SetRPCD_CharIsWellRounded(i, newValue)
	self.db.global.RPCD_CharIsWellRounded = newValue
	RP_CombatDice:SaySetSkills()
end

--Function for improved substr
function RP_CombatDice:isubstr(string, start, length)
  if not string then
      return ''
  end
  -- Sanity checks: make sure integers are integers.
  start = floor(tonumber(start)) or 1
  if length == nil then
      length = strlen(string)
  end
  if length < 0 and start < 0 and abs(length) > abs(start) then
      return false
  end
  if start < 0 then
      start = strlen(string) + (start + 1)
  end
  length = floor(tonumber(length))
  local String = ''
  if length >= 0 then
      String = strsub(string, start, (start - 1) + length)
  else
      String = string
      String = strsub(String, start)
      String = strsub(String, 1, strlen(String) + length)
  end
  return String
end

--Function for the /RPCD command:
function RP_CombatDice:DoRP_CombatDice(msg)

  --RP_CombatDice:Print("debug: DoRP_CombatDice msg=["..msg.."]")

	if msg ~= "" then
		--If message is blank, don't annoy the local group
    -- /RPCD l investigating the metal monster

    task =  RP_CombatDice:isubstr(msg, 1, 1)
    reason = RP_CombatDice:isubstr(msg, 2, 199)

    --set basic reason word
    local reason_word = "unknown"

    if task == 'r' then
      reason_word = "Ranged Combat"
    end
    if task == 'm' then
      reason_word = "Melee Combat"
    end
    if task == 's' then
      reason_word = "Spell Combat"
    end
    if task == 'h' then
      reason_word = "Healing"
    end
    if task == 'l' then
      reason_word = "Lore"
    end

    --set up base modifiers condition
		local modifier_list = ""
		local modifer_value = 0

		--throw some dice
		local die1 = math.random(1,10)
		local die2 = math.random(1,10)
		local baseRoll = die1 + die2

		--Add applicable modifiers
    if self.db.global.RPCD_CharIsWellRounded then
      modifier_list = modifier_list.."+1(Well Rounded) "
      modifer_value = modifer_value + 1
    end

    if self.db.global.RPCD_CharIsMeleeCombat and task == 'm' then
      modifier_list = modifier_list.."+2(Melee Combat) "
      modifer_value = modifer_value + 2
    end

    if self.db.global.RPCD_CharIsRangedCombat and task == 'r' then
      modifier_list = modifier_list.."+2(Ranged Combat) "
      modifer_value = modifer_value + 2
    end

    if self.db.global.RPCD_CharIsSpellCombat and task == 's' then
      modifier_list = modifier_list.."+2(Spell Combat) "
      modifer_value = modifer_value + 2
    end

    if self.db.global.RPCD_CharIsHealer and task == 'h' then
      modifier_list = modifier_list.."+2(Healer) "
      modifer_value = modifer_value + 2
    end

    if self.db.global.RPCD_CharIsLore and task == 'r' then
      modifier_list = modifier_list.."+2(Lore & History) "
      modifer_value = modifer_value + 2
    end

		--Build text string to use for the emote.
		local emotestring = "makes a roll of "..baseRoll
		local totalRoll = 0
	  totalRoll = baseRoll + modifer_value

		if modifer_value ~= 0 then
		  --if we have a DM, factor it in
		  emotestring = emotestring..modifier_list
	  else
	    emotestring = emotestring.." with no modifiers "
	  end

	  --tack on the total
		emotestring = emotestring.." for a total of "..totalRoll.." on a "..reason_word.." task for "..reason
		-- |cff00ffff text |r

		--check for special effect
		if totalRoll <= 3 then
		  emotestring = "*CRITFAIL* :"..emotestring
		  --|cffff3300 text |r
	  end
	  if totalRoll >= 20 then
		  emotestring = "CRITSUCCESS* :"..emotestring
		  -- "|cff33cc33*CRITSUCCESS*|r
	  end

		--Send the emote.
		SendChatMessage(emotestring, "EMOTE")

	else
		--no message, handle that
		RP_CombatDice:NoMsg()
	end
end

--Function for blank message handling
function RP_CombatDice:NoMsg()
  --pop a reminder.
  if self.db.global.RPCDsaidhelp == 0 then
    RP_CombatDice:Print("You might have forgotten roll description.")
    RP_CombatDice:Print("Use:  /RPCD <task> <reason>")
    RP_CombatDice:Print("'task' must be one of 'm, 'r', 's', 'h', or 'l'")
    self.db.global.RPCDsaidhelp = 1
  end

  --Define text to use for the emote.
  local emotestring = self.db.global.RPCDnull

  --Send the emote.
  SendChatMessage(emotestring, "EMOTE")
end

--Function to print current enabled skills
function RP_CombatDice:SaySetSkills()
  local activeSkills = ""

  if self.db.global.RPCD_CharIsWellRounded then
    activeSkills = activeSkills.." [Well Rounded]"
  end

  if self.db.global.RPCD_CharIsMeleeCombat then
    activeSkills = activeSkills.." [Melee Fighter]"
  end

  if self.db.global.RPCD_CharIsRangedCombat then
    activeSkills = activeSkills.." [Ranged Fighter]"
  end

  if self.db.global.RPCD_CharIsSpellCombat then
    activeSkills = activeSkills.." [Spell Fighter]"
  end

  if self.db.global.RPCD_CharIsHealer then
    activeSkills = activeSkills.." [Healer]"
  end

  if self.db.global.RPCD_CharIsLore then
    activeSkills = activeSkills.." [Lore]"
  end

  RP_CombatDice:Print("Profiles active: "..activeSkills)

end

--Function to give a bried explaination
function RP_CombatDice:DoRP_SayDiffList()
  RP_CombatDice:Print("9+ is easy for an unskilled person")
  RP_CombatDice:Print("11+ is challenging for an unskilled person")
  RP_CombatDice:Print("13+ is very difficult for an unskilled person")
  RP_CombatDice:Print("A roll of 2 or 3 is a 'fumble'")
  RP_CombatDice:Print("A roll of 20+ is a 'critical success'")
end

--Initialization Code
function RP_CombatDice:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RP_CombatDice3DB", defaults)

	AceConfig:RegisterOptionsTable("RP Combat Dice", options)
	local configwindow = AceConfigDialog:AddToBlizOptions("RP Combat Dice")

	--set the help-test default on every reload
  self.db.global.RPCDsaidhelp = 0

	--Register Slash Command for /RPCDcfg to open configuration screen
	SlashCmdList["RPCDCFG"] = function() InterfaceOptionsFrame_OpenToCategory(configwindow) end
	SLASH_RPCDCFG1 = "/RPCDcfg"

	--Register Slash Command for /RPCD to call DoRP_CombatDice.
	SlashCmdList["RPCD"] = function(msg) self:DoRP_CombatDice(msg) end
	SLASH_RPCD1 = "/RPCD"
	SLASH_RPCD2 = "/rproll"

  --Register ST help blurb
  SlashCmdList["RPCDDIFF"] = function() self:DoRP_SayDiffList() end
  SLASH_RPCDDIFF1 = "/rpdiff"

  --Let the user know we're GTG
	RP_CombatDice:Print("Loaded OK.")
	RP_CombatDice:Print("   make skill roll: /rproll <task> <roll description>")
	RP_CombatDice:Print("   set your skills: /RPCDcfg")
	RP_CombatDice:Print("   explain difficulties: /rpdiff")
	RP_CombatDice:SaySetSkills()

end
